package org.artifactory.probes.model;

import lombok.Builder;

/**
 * Readiness status object to store all the readiness checks result.
 *
 * @author Andrey Tashlitsky
 */
@Builder
public class ReadinessStatus {

    /**
     * True if Artifactory is in RUNNING state. All other states (CONVERTING, STARTING, etc) are considered "false".
     */
    private final boolean running;
    /**
     * True if Artifactory application context is in Offline state.
     */
    private final boolean offline;
    /**
     * True if Artifactory can write to disk.
     */
    private final boolean canWrite;
    /**
     * True if the storage service (binary provider) is ready.
     */
    private final boolean storageServiceReady;
    /**
     * True if the DB service is ready.
     */
    private final boolean internalDbServiceHealthy;
    /**
     * True if Access service is ready.
     */
    private final boolean accessServiceReady;

    /**
     * Returns the readiness status.
     *
     * @return True if all the conditions for readiness are met. False otherwise.
     */
    public boolean isReady() {
        return running && !offline && canWrite && storageServiceReady && internalDbServiceHealthy && accessServiceReady;
    }
}
