package org.artifactory.ph;

import org.artifactory.common.repo.PackageType;
import org.artifactory.descriptor.repo.RepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.repo.Repo;
import org.testng.annotations.Test;

import java.util.EnumSet;

import static org.artifactory.ph.PackageRepoTypeMapper.EXCLUDED_REPO_TYPES;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;

@Test
public class PackageRepoTypeMapperTest {

    @Test
    public void testPackageType() {
        for (RepoType rt : EnumSet.complementOf(EXCLUDED_REPO_TYPES)) {
            PackageType pt = PackageRepoTypeMapper.packageType(rt);
            assertNotNull(pt,
                    "No " + PackageType.class.getSimpleName() + " defined to " + RepoType.class.getSimpleName() + ": " +
                            rt);
            assertEquals(pt, PackageType.valueOf(rt.name().toUpperCase()));
        }
    }

    @Test
    public void testExcludedPackages() {
        for (RepoType rt : EXCLUDED_REPO_TYPES) {
            PackageType pt = PackageRepoTypeMapper.packageType(rt);
            assertNull(pt, rt + " should return null");
        }
    }

    @Test
    public void testTestPackageType() {
        Repo repo = mock(Repo.class);
        RepoDescriptor descriptor = mock(RepoDescriptor.class);
        when(descriptor.getType()).thenReturn(RepoType.CRAN);
        when(repo.getDescriptor()).thenReturn(descriptor);
        assertEquals(PackageRepoTypeMapper.packageType(repo), PackageType.CRAN);
    }
}