package org.artifactory.queue.register;

import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.testng.annotations.Test;
import org.testng.collections.Maps;

import java.util.Map;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class PersistentQueueRepositoryInterceptorTest {

    @Test
    public void testOnAfterSave() {
        PersistentQueueRegistrationService queueRegistrationService = mock(PersistentQueueRegistrationService.class);
        CentralConfigDescriptor newDescriptor = mock(CentralConfigDescriptor.class);
        prepareDescriptor(newDescriptor, "key");
        CentralConfigDescriptor oldDescriptor = mock(CentralConfigDescriptor.class);
        prepareDescriptor(oldDescriptor, "key_old");

        PersistentQueueRepositoryInterceptor interceptor =
                new PersistentQueueRepositoryInterceptor(queueRegistrationService);
        interceptor.onAfterSave(newDescriptor, oldDescriptor);
        verify(queueRegistrationService, times(1)).deRegisterRepository("key_old");
        verify(queueRegistrationService, times(1)).registerRepository("key");
    }

    private void prepareDescriptor(CentralConfigDescriptor descriptor, String key) {
        Map<String, FederatedRepoDescriptor> federatedRepoDescriptorMap = Maps.newLinkedHashMap();
        FederatedRepoDescriptor federatedRepoDescriptor = new FederatedRepoDescriptor();
        federatedRepoDescriptor.setKey(key);
        federatedRepoDescriptorMap.put(federatedRepoDescriptor.getKey(), federatedRepoDescriptor);
        when(descriptor.getFederatedRepositoriesMap()).thenReturn(federatedRepoDescriptorMap);
    }
}