package org.artifactory.repo.service.flexible.tools;

import org.artifactory.api.common.MoveMultiStatusHolder;
import org.artifactory.repo.LocalRepo;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoRepoPath;
import org.artifactory.repo.StoringRepo;
import org.artifactory.storage.db.fs.session.SqlStorageSession;
import org.artifactory.storage.fs.MutableVfsItem;
import org.artifactory.storage.fs.lock.SessionLockEntry;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.artifactory.repo.service.flexible.tools.MoveCopyTestUtils.prepareCopyOrMoveOverrideMocks;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

public class MoveCopyHelperTest {

    @Mock
    RepoPath repoPathMock;
    @Mock
    LocalRepo repoMock;
    @Mock
    MutableVfsItem mutableVfsItemMock;
    @Mock
    SqlStorageSession sqlStorageSessionMock;
    @Mock
    SessionLockEntry sessionLockEntryMock;
    @Mock
    MoveMultiStatusHolder moveMultiStatusHolderMock;

    @BeforeMethod
    public void setup() {
        MockitoAnnotations.initMocks(this);
    }

    @Test(dataProvider = "overrideTargetFileProvider")
    public void testOverrideTargetFileIfExist(boolean returnTargetItem,
            boolean basicStatusHolderErrorLog,
            int saveInvokeNum,
            int setWriteInvokeNum,
            int deleteInvokeNum,
            int createOrGetFileInvokeNum,
            int statusHolderErrorInvokeNum) {

        RepoRepoPath<StoringRepo> localRepoRepoRepoPath =
                prepareCopyOrMoveOverrideMocks(repoPathMock, repoMock, mutableVfsItemMock, sessionLockEntryMock,
                        sqlStorageSessionMock, moveMultiStatusHolderMock, returnTargetItem);

        MoveCopyHelper
                .overrideTargetFileIfExist(localRepoRepoRepoPath, repoMock, moveMultiStatusHolderMock,
                        basicStatusHolderErrorLog);

        verify(sessionLockEntryMock, times(saveInvokeNum)).save();
        verify(sessionLockEntryMock, times(setWriteInvokeNum)).setWriteFsItem(any());
        verify(mutableVfsItemMock, times(deleteInvokeNum)).delete(any());
        verify(repoMock, times(createOrGetFileInvokeNum)).createOrGetFile(any());
        verify(moveMultiStatusHolderMock, times(statusHolderErrorInvokeNum)).error(any(), any());
    }

    @DataProvider
    public Object[][] overrideTargetFileProvider() {
        return new Object[][] {
                {true, false, 1, 2, 1, 1, 0},
                {false, true, 0, 0, 0, 0, 1}
        };
    }
}