package org.artifactory.traffic;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.common.ConstantValues;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.projects.ProjectsService;
import org.artifactory.storage.db.servers.service.ArtifactoryServersCommonService;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.traffic.v2.TrafficEntryV2;
import org.mockito.Mockito;
import org.testng.annotations.Test;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNull;

public class TrafficServiceAddProjectTest extends ArtifactoryHomeBoundTest {

    @Test
    public void smoke() {
        ArtifactorySystemProperties artifactoryProperties = ArtifactoryHome.get().getArtifactoryProperties();
        artifactoryProperties.setProperty(ConstantValues.trafficCollectionActive.getPropertyName(), "true");
        artifactoryProperties.setProperty(ConstantValues.trafficVersion2Enabled.getPropertyName(), "true");
        artifactoryProperties.setProperty(ConstantValues.trafficVersion2EnabledFinally.getPropertyName(), "true");
        ArtifactoryServersCommonService serversService = Mockito.mock(ArtifactoryServersCommonService.class);
        RepositoryService repositoryService = Mockito.mock(RepositoryService.class);
        ProjectsService projectsService = Mockito.mock(ProjectsService.class);
        when(projectsService.getProjectKeyOfRepository(any())).thenReturn("projectname");
        AuthorizationService authorizationService = Mockito.mock(AuthorizationService.class);
        when(authorizationService.currentUsername()).thenReturn("cool-user");
        TrafficServiceForTest trafficService =
                new TrafficServiceForTest(serversService, repositoryService, projectsService, authorizationService);
        TrafficEntryV2 entryV2 = TrafficEntryV2.builder()
                .repoPath("a:b")
                .action(TrafficEntryV2.TrafficAction.DOWNLOAD)
                .build();

        trafficService.handleTrafficEntry(entryV2);

        assertNull(trafficService.loggedEntryWithoutExtendedInfo);
        assertEquals(trafficService.loggedEntryWithExtendedInfo.getProjectKey(), "projectname");
        assertEquals(trafficService.loggedEntryWithExtendedInfo.getRepoKey(), "a");
        assertEquals(trafficService.loggedEntryWithExtendedInfo.getUsername(), "cool-user");
    }

    private static class TrafficServiceForTest extends TrafficServiceImpl {

        private TrafficEntryV2 loggedEntryWithExtendedInfo;
        private TrafficEntryV2 loggedEntryWithoutExtendedInfo;

        public TrafficServiceForTest(ArtifactoryServersCommonService serversService,
                RepositoryService repositoryService,
                ProjectsService projectsService,
                AuthorizationService authorizationService) {
            super();
            setServersService(serversService);
            setRepositoryService(repositoryService);
            setProjectsService(projectsService);
            setAuthorizationService(authorizationService);
        }

        @Override
        protected void handleMetrics(String repoName, String name) {
            // empty
        }

        @Override
        protected void logEntryWithExtendedInfo(TrafficEntryV2 entry) {
            this.loggedEntryWithExtendedInfo = entry;
        }

        @Override
        protected void logEntry(TrafficEntryV2 entry) {
            this.loggedEntryWithoutExtendedInfo = entry;
        }
    }
}
