package org.artifactory.event.queue;

import javax.annotation.Nonnull;

/**
 * Can be thrown by the QueueEventImplementor to know which event couldn't proceed
 * this allows us to advance the marker up until the point of failure instead of not advancing at all.
 *
 * @author avishayh
 */
public class QueueEventException extends RuntimeException {

    private final QueueEvent failedEvent;
    private final boolean omitError;

    public QueueEventException(@Nonnull QueueEvent queueEvent, String errorMessage) {
        this(queueEvent, errorMessage, false);
    }

    public QueueEventException(@Nonnull QueueEvent queueEvent, String errorMessage, boolean omitError) {
        super(errorMessage);
        this.failedEvent = queueEvent;
        this.omitError = omitError;
    }

    @Nonnull
    public QueueEvent getFailedEvent() {
        return failedEvent;
    }

    public boolean isOmitError() {
        return omitError;
    }

    @Override
    public String toString() {
        String s = getClass().getName();
        String message = getMessage();
        return String.format("[%s] %s: %s", failedEvent, s, message);
    }
}
