package org.artifactory.core.api.ph.service;

import org.artifactory.core.api.ph.exception.PackageRepoDescriptorException;

import javax.annotation.Nullable;
import javax.servlet.http.HttpServletRequest;


public interface PackageArtifactoryConfigService {

    /**
     * Get String value from config descriptor, by key name
     *
     * @param repoKey the repository key
     * @param key the repo descriptor key name
     * @param def default value to return if not exists
     * @throws PackageRepoDescriptorException if the repoKey doesn't exist OR if the key is of an inappropriate type
     * @return configuration value
     */
    String getStringFromRepoConfig(String repoKey, String key, String def) throws PackageRepoDescriptorException;

    /**
     * Get int value from config descriptor, by key name
     *
     * @param repoKey the repository key
     * @param key the repo descriptor key name
     * @param def default value to return if not exists
     * @throws PackageRepoDescriptorException if the repoKey doesn't exist OR if the key is of an inappropriate type
     * @return configuration value
     */
    int getIntFromRepoConfig(String repoKey, String key, int def) throws PackageRepoDescriptorException;

    /**
     * Get boolean value from config descriptor, by key name
     *
     * @param repoKey the repository key
     * @param key the repo descriptor key name
     * @param def default value to return if not exists
     * @throws PackageRepoDescriptorException if the repoKey doesn't exist OR if the key is of an inappropriate type
     * @return configuration value
     */
    boolean getBooleanFromRepoConfig(String repoKey, String key, boolean def) throws PackageRepoDescriptorException;

    /**
     * Get long value from config descriptor, by key name
     *
     * @param repoKey the repository key
     * @param key the repo descriptor key name
     * @param def default value to return if not exists
     * @throws PackageRepoDescriptorException if the repoKey doesn't exist OR if the key is of an inappropriate type
     * @return configuration value
     */
    long getLongFromRepoConfig(String repoKey, String key, long def) throws PackageRepoDescriptorException;

    /**
     * Return Artifactory Base url from headers, repo descriptor or http request
     * Artifactory base url result is trim trailing slashes. E.g: https://lala.land.com/artifactory
     * @param httpRequest The httpRequest context
     * @return Artifactory Base url
     */
    String getArtifactoryBaseUrl(@Nullable HttpServletRequest httpRequest);

}
