package org.artifactory.storage.db.security.service;

import org.artifactory.test.TestUtils;
import org.testng.annotations.Test;

import java.util.concurrent.Callable;
import java.util.concurrent.atomic.AtomicInteger;

import static org.testng.Assert.assertEquals;

/**
 * @author Shay Bagants
 */
@Test
public class VersioningCacheTest {

    public void testVersionCache() {
        AtomicInteger timesCallableCalled = new AtomicInteger(0);
        Callable<BasicCacheModel> callable = getCacheCallable(timesCallableCalled);
        VersionCache<BasicCacheModel> versioningCacheImpl = new VersioningCacheImpl<>(20000, callable);
        // call the cache twice, ensure it never reached the callable more than once
        versioningCacheImpl.get();
        BasicCacheModel cache = versioningCacheImpl.get();
        assertEquals(cache.getVersion(), 1);
        // validate callable called only once
        assertEquals(timesCallableCalled.get(), 1);

        versioningCacheImpl.promoteVersion();
        // call the cache twice, ensure it never reached the callable more than once
        versioningCacheImpl.get();
        cache = versioningCacheImpl.get();
        assertEquals(cache.getVersion(), 2);
        // validate callable called only 2 times
        assertEquals(timesCallableCalled.get(), 2);

        // promote the cache 10 more times
        TestUtils.repeat(10, versioningCacheImpl::promoteVersion);
        // call the cache twice, ensure it never reached the callable more than once
        versioningCacheImpl.get();
        cache = versioningCacheImpl.get();
        assertEquals(cache.getVersion(), 12);
        // validate callable called 3 times
        assertEquals(timesCallableCalled.get(), 3);
    }

    @Test(expectedExceptions = VersioningCacheException.class, expectedExceptionsMessageRegExp = "^Failed to reload " +
            "cache\\.$")
    public void testVersionCacheExceptionHandling() {
        VersionCache<BasicCacheModel> versioningCacheImpl =
                new VersioningCacheImpl<>(20000, getCacheCallableThrowsError());
        versioningCacheImpl.get();
    }

    private Callable<BasicCacheModel> getCacheCallable(AtomicInteger atomicInteger) {
        return () -> {
            atomicInteger.incrementAndGet();
            return new CacheModel();
        };
    }

    private Callable<BasicCacheModel> getCacheCallableThrowsError() {
        return () -> {
            throw new RuntimeException("Some sensitive data");
        };
    }

    public static class CacheModel implements BasicCacheModel {
        long version = 0;

        @Override
        public long getVersion() {
            return version;
        }

        @Override
        public void setVersion(long ver) {
            version = ver;
        }

        @Override
        public void destroy() {

        }
    }
}